@@ -1,7 +1,7 @@
 /* GNU cmp - compare two files byte by byte
 
    Copyright (C) 1990-1996, 1998, 2001-2002, 2004, 2006-2007, 2009-2013,
-   2015-2021 Free Software Foundation, Inc.
+   2015-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@
 
 static int cmp (void);
 static off_t file_position (int);
-static size_t block_compare (word const *, word const *) _GL_ATTRIBUTE_PURE;
+static size_t block_compare (word const *, word const *) ATTRIBUTE_PURE;
 static size_t count_newlines (char *, size_t);
 static void sprintc (char *, unsigned char);
 
@@ -110,8 +110,7 @@ static struct option const long_options[] =
   {0, 0, 0, 0}
 };
 
-static void try_help (char const *, char const *) __attribute__((noreturn));
-static void
+static _Noreturn void
 try_help (char const *reason_msgid, char const *operand)
 {
   if (reason_msgid)
@@ -390,7 +389,7 @@ cmp (void)
   char *buf1 = (char *) buffer1;
   int differing = 0;
   int f;
-  int offset_width IF_LINT (= 0);
+  int offset_width IF_LINT (= 0); /* IF_LINT due to GCC bug 101768.  */
 
   if (comparison_type == type_all_diffs)
     {
