@@ -1,7 +1,7 @@
 /* GNU diff - compare files line by line
 
    Copyright (C) 1988-1989, 1992-1994, 1996, 1998, 2001-2002, 2004, 2006-2007,
-   2009-2013, 2015-2021 Free Software Foundation, Inc.
+   2009-2013, 2015-2023 Free Software Foundation, Inc.
 
    This file is part of GNU DIFF.
 
@@ -33,7 +33,6 @@
 #include <fnmatch.h>
 #include <getopt.h>
 #include <hard-locale.h>
-#include <prepargs.h>
 #include <progname.h>
 #include <sh-quote.h>
 #include <stat-time.h>
@@ -73,7 +72,7 @@ static void summarize_regexp_list (struct regexp_list *);
 static void specify_style (enum output_style);
 static void specify_value (char const **, char const *, char const *);
 static void specify_colors_style (char const *);
-static void try_help (char const *, char const *) __attribute__((noreturn));
+static _Noreturn void try_help (char const *, char const *);
 static void check_stdout (void);
 static void usage (void);
 
@@ -246,7 +245,11 @@ option_list (char **optionvec, int count)
   char *p;
 
   for (i = 0; i < count; i++)
-    size += 1 + shell_quote_length (optionvec[i]);
+    {
+      size_t optsize = 1 + shell_quote_length (optionvec[i]);
+      if (INT_ADD_WRAPV (optsize, size, &size))
+	xalloc_die ();
+    }
 
   p = result = xmalloc (size);
 
@@ -379,21 +382,60 @@ main (int argc, char **argv)
         case 'D':
           specify_style (OUTPUT_IFDEF);
           {
-            static char const C_ifdef_group_formats[] =
-              "%%=%c#ifndef %s\n%%<#endif /* ! %s */\n%c#ifdef %s\n%%>#endif /* %s */\n%c#ifndef %s\n%%<#else /* %s */\n%%>#endif /* %s */\n";
-            char *b = xmalloc (sizeof C_ifdef_group_formats
-                               + 7 * strlen (optarg) - 14 /* 7*"%s" */
-                               - 8 /* 5*"%%" + 3*"%c" */);
-            sprintf (b, C_ifdef_group_formats,
-                     0,
-                     optarg, optarg, 0,
-                     optarg, optarg, 0,
-                     optarg, optarg, optarg);
-            for (i = 0; i < sizeof group_format / sizeof group_format[0]; i++)
-              {
-                specify_value (&group_format[i], b, "-D");
-                b += strlen (b) + 1;
-              }
+	    static char const C_ifdef_group_formats[]
+	      = (/* UNCHANGED */
+		 "%="
+		 "\0"
+
+		 /* OLD */
+		 "#ifndef @\n"
+		 "%<"
+		 "#endif /* ! @ */\n"
+		 "\0"
+
+		 /* NEW */
+		 "#ifdef @\n"
+		 "%>"
+		 "#endif /* @ */\n"
+		 "\0"
+
+		 /* CHANGED */
+		 "#ifndef @\n"
+		 "%<"
+		 "#else /* @ */\n"
+		 "%>"
+		 "#endif /* @ */\n");
+
+	    size_t alloc = strlen (optarg);
+	    if (INT_MULTIPLY_WRAPV (alloc, 7, &alloc)
+		|| INT_ADD_WRAPV (alloc,
+				  sizeof C_ifdef_group_formats - 7 /* 7*"@" */,
+				  &alloc))
+	      xalloc_die ();
+	    char *b = xmalloc (alloc);
+	    char *base = b;
+	    int changes = 0;
+
+	    for (i = 0; i < sizeof sizeof C_ifdef_group_formats; i++)
+	      {
+		char ch = C_ifdef_group_formats[i];
+		switch (ch)
+		  {
+		  default:
+		    *b++ = ch;
+		    break;
+
+		  case '@':
+		    b = stpcpy (b, optarg);
+		    break;
+
+		  case '\0':
+		    *b++ = ch;
+		    specify_value (&group_format[changes++], base, "-D");
+		    base = b;
+		    break;
+		  }
+	      }
           }
           break;
 
@@ -686,6 +728,9 @@ main (int argc, char **argv)
       time_format = "%Y-%m-%d %H:%M:%S.%N %z";
 #else
       time_format = "%Y-%m-%d %H:%M:%S %z";
+#endif
+#if !HAVE_TM_GMTOFF
+      localtz = tzalloc (getenv ("TZ"));
 #endif
     }
   else
@@ -750,8 +795,12 @@ main (int argc, char **argv)
       if (!group_format[UNCHANGED])
         group_format[UNCHANGED] = "%=";
       if (!group_format[CHANGED])
-        group_format[CHANGED] = concat (group_format[OLD],
-                                        group_format[NEW], "");
+	{
+	  char *p = xmalloc (strlen (group_format[OLD])
+			     + strlen (group_format[NEW]) + 1);
+	  group_format[CHANGED] = p;
+	  strcpy (stpcpy (p, group_format[OLD]), group_format[NEW]);
+	}
     }
 
   no_diff_means_no_output =
@@ -807,7 +856,7 @@ main (int argc, char **argv)
   print_message_queue ();
 
   check_stdout ();
-  exit (exit_status);
+  cleanup_signal_handlers ();
   return exit_status;
 }
 
@@ -1336,11 +1385,11 @@ compare_files (struct comparison const *parent,
           /* We have two files that are not to be compared.  */
 
           /* See POSIX 1003.1-2001 for this format.  */
-          message5 ("File %s is a %s while file %s is a %s\n",
-                    file_label[0] ? file_label[0] : cmp.file[0].name,
-                    file_type (&cmp.file[0].stat),
-                    file_label[1] ? file_label[1] : cmp.file[1].name,
-                    file_type (&cmp.file[1].stat));
+          message ("File %s is a %s while file %s is a %s\n",
+		   file_label[0] ? file_label[0] : cmp.file[0].name,
+		   file_type (&cmp.file[0].stat),
+		   file_label[1] ? file_label[1] : cmp.file[1].name,
+		   file_type (&cmp.file[1].stat));
 
           /* This is a difference.  */
           status = EXIT_FAILURE;
@@ -1386,11 +1435,11 @@ compare_files (struct comparison const *parent,
           /* We have two files that are not to be compared, because
              one of them is a symbolic link and the other one is not.  */
 
-          message5 ("File %s is a %s while file %s is a %s\n",
-                    file_label[0] ? file_label[0] : cmp.file[0].name,
-                    file_type (&cmp.file[0].stat),
-                    file_label[1] ? file_label[1] : cmp.file[1].name,
-                    file_type (&cmp.file[1].stat));
+          message ("File %s is a %s while file %s is a %s\n",
+		   file_label[0] ? file_label[0] : cmp.file[0].name,
+		   file_type (&cmp.file[0].stat),
+		   file_label[1] ? file_label[1] : cmp.file[1].name,
+		   file_type (&cmp.file[1].stat));
 
           /* This is a difference.  */
           status = EXIT_FAILURE;
